        /********************************************************
         *                                                      *
         *      ARTIFICIAL   NEURAL   NETWORK   SOFTWARE        *
         *                                                      *
         *      By Andrea Hadjiprocopis                         *
         *                                                      *
         *              1992/93                                 *
         *                                                      *
         ********************************************************/

#include "XMain.h"
#include "SMain.h"


/* Specify format for saving and loading configuration files as follows:
1	%The total number of layers
2	^The number of nodes in every layer from input to output
3	>The Error Associated with it
.	
.
.
3	!layer		is int
4	@node		is int
5	#threshold	is double
6	$weight		is double	
	weight connecting current node with previous node according to line
	.
	.
	.
	weight connecting current node with the last node of the previous layer
i	!layer
i+1	@node
i+2	#threshold
	etc..
*/



void	FileopsPopupRoutine( Widget W, XtPointer client_data, XtPointer call_data )
{
        Arg	arg[10];
        int	i;

if( Network_Not_Mapped) {
        ErrorHandler(44, NOT_MAPPED_YET, 0);
        return;
}
	i = 0;
        XtSetArg(arg[i], XtNwidth, (Dimension )/*196*/ 250 );i++;
        XtSetArg(arg[i], XtNheight, (Dimension )100);i++;
        XtSetArg(arg[i], XtNx, (Position )50);i++;
        XtSetArg(arg[i], XtNy, (Position )50);i++;
	XtSetValues( FileopsPopup, arg, i );

	XtPopupSpringLoaded( FileopsPopup );

}

void	FileopsQuitRoutine( Widget W, XtPointer client_data, XtPointer call_data )
{
	XtPopdown( FileopsPopup ); 
}



void 	FileopsSaveParamsRoutine( Widget W, XtPointer client_data, XtPointer call_data )
/************************************************************************
 * This is a function to SAVE the configuration, it is called from main	*
 * so it will know the NCP and CFTable and all other globals		*
 * It is called once and will SAVE the configuartion of the whole net	*
 ************************************************************************/
{
	FILE 	*file;
	Arg	arg[2];
	String	string;
	char	filename[70];
	int 	i,j,k;/* i=layer, j=current node */
	

        XtSetArg(arg[0], XtNstring, &string);
        XtGetValues(FileopsNameBox, arg, 1);
        strcpy(filename, string);
	if( (file=fopen(filename,"w+")) == NULL ){
		ErrorHandler(77, FILE_OPEN_ERROR, 0);
		return;
	}

	fprintf(file, "%%%d\n", max_layers);
	for(i=0;i<max_layers;i++)	fprintf(file,"^%d\n",CFTable[i]);


	for(i=0;i<max_layers;i++)	/* for every layer */
		for(j=0;j<CFTable[i];j++){ /* for every node in ith layer */
			fprintf(file,"!%d\n@%d\n#%lf\n",i,j,NCP[i][j]->threshold);/* print the layer and the node */
			if(i==0) {
				fprintf(file,"1.0\n");
				continue;
			}
			if( i > 0 )	for(k=0;k<CFTable[i-1];k++) /*for every previous node*/
						fprintf(file, "%lf\n", NCP[i][j]->weight[k]);
		}

	fclose( file );
/* thats that */
}


void	FileopsLoadParamsRoutine( Widget W, XtPointer client_data, XtPointer call_data )
/************************************************************************
 * This is a function to load the configuration, it is called from main	*
 * so it will know the NCP and CFTable and all other globals		*
 * It is called once and will load the configuartion of the whole net	*
 * GIVEN that the network topology is exactly the same			*
 ************************************************************************/
{
	FILE 	*file;
        Arg     arg[2];
        String  string;
        char    filename[70], *s1, *s2;   
	int 	i,j,k;/* i=layer, j=current node */


	if( (s1=(char *)calloc((CFTable[0]+CFTable[max_layers-1]+1)*50, sizeof(char))) == NULL )
		ErrorHandler(78, MALLOC_ERROR, 0);
	if( (s2=(char *)calloc((CFTable[0]+CFTable[max_layers-1]+1)*50, sizeof(char))) == NULL )
		ErrorHandler(78, MALLOC_ERROR, 1);


        XtSetArg(arg[0], XtNstring, &string);
        XtGetValues(FileopsNameBox, arg, 1);
        strcpy(filename, string); 
 	if( filename[0] == NULL ) strcpy(filename, "DATA.PAR");

	if( (file=fopen(filename,"r")) == NULL ){
		ErrorHandler(78, FILE_OPEN_ERROR, 0);
		return;
	}

	fgets(s1, SAFE_LINE, file);
	if( (s1[0] != '%' ) || (atoi(++s1) != max_layers) )
		ErrorHandler(78, WRONG_PARAMETERS, -1);
	for(i=0;i<max_layers;i++){
		fgets(s1, SAFE_LINE, file);
		if( (s1[0] != '^' ) || (atoi(++s1) != CFTable[i]) )
			ErrorHandler(78, WRONG_PARAMETERS, i);	
	} /* so much for checking topolgy */

	for(i=0;i<max_layers;i++) /* for every layer */
		for(j=0;j<CFTable[i];j++){ /* for every node in the ith layer*/
			fgets(s1, SAFE_LINE, file);
			fgets(s2, SAFE_LINE, file);
			if( (s1[0] != '!') || (s2[0] != '@') || (atoi(++s1) != i) || (atoi(++s2) != j) )
				ErrorHandler(78, WRONG_PARAMETERS, 1000+i);
			fgets(s1, SAFE_LINE, file);
			if( s1[0] != '#' )
				ErrorHandler(78, WRONG_PARAMETERS, 2000+i);
			(NCP[i][j]->threshold) = atof(++s1);
			if(i==0) {
				fgets(s1, SAFE_LINE, file);
				if( atof(s1) != 1.0 )
					ErrorHandler(78, WRONG_PARAMETERS, 3000+i); /*just to move the file pointer */
				(NCP[i][j]->weight[0]) = 1.0;
			} else {
				for(k=0;k<CFTable[i-1];k++){ /*for every node in prev l*/
					fgets(s1, SAFE_LINE, file);
					(NCP[i][j]->weight[k]) = atof(s1); /* no need to plus plus since weights have no char in front */
					if( feof( file ) ) 
						ErrorHandler(78, UNEXPECTED_EOF, 0);
					
				} /* for k */
			} /* if i == 0 */
		} /* for j */

	free(s1);
}
	

/* **************************************** TRAINING *************************************************/


void 	FileopsSaveTrainingRoutine( Widget W, XtPointer client_data, XtPointer call_data )
/***********************************************************************
 * This is the function which will save the training vectors in a file *
 * For your convenience and only...				       *
 ***********************************************************************/
{
	FILE 	*file;
	Neuron 	n;
	Arg	arg[2];
	String	string;
	char	filename[70];
	int 	i,j,k;/* i=layer, j=current node */
	

        XtSetArg(arg[0], XtNstring, &string);
        XtGetValues(FileopsNameBox, arg, 1);
        strcpy(filename, string);

	if( (n=(Neuron )calloc(1, sizeof(struct NEURON))) == NULL )
		ErrorHandler(79, MALLOC_ERROR, 0);

	if( (file=fopen(filename,"w+")) == NULL ){
		ErrorHandler(79, FILE_OPEN_ERROR, 0);
		return;
	}

	fprintf(file, "%%%d\n", num_exemplars);

	for(i=0;i<num_exemplars;i++){
		for(j=0;j<CFTable[0];j++)
			fprintf(file, "%lf ", Training[i].Input[j]);
		fputs("     \0", file);
		for(k=0;k<CFTable[max_layers-1];k++)
			fprintf(file, "%lf ", Training[i].Target[k]);
		fputs("\n", file);
	}
	
	fclose( file );
/* thats that */
}


void	FileopsLoadTrainingRoutine( Widget W, XtPointer client_data, XtPointer call_data )
/************************************************************************
 * This is a function to load the configuration, it is called from main	*
 * so it will know the NCP and CFTable and all other globals		*
 * It is called once and will load the configuartion of the whole net	*
 * GIVEN that the network topology is exactly the same			*
 ************************************************************************/
{
	FILE 	*file;
        Arg     arg[2];
        String  string;
        char    filename[70], *s1;   
	int 	i,j;/* i=layer, j=current node */

        XtSetArg(arg[0], XtNstring, &string);
        XtGetValues(FileopsNameBox, arg, 1);
        strcpy(filename, string); 
 	if( filename[0] == NULL ) strcpy(filename, "DATA.TRA");

	if( (s1=(char *)calloc((CFTable[0]+CFTable[max_layers-1]+1)*50, sizeof(char))) == NULL )
		ErrorHandler(80, MALLOC_ERROR, 0);

	if( (file=fopen(filename,"r")) == NULL ){
		ErrorHandler(80, FILE_OPEN_ERROR, 0);
		return;
	}

	fgets(s1, SAFE_LINE, file);
	if( (s1[0] != '%' ) || (atoi(++s1) != num_exemplars) )
		ErrorHandler(80, WRONG_PARAMETERS, -1);
	
	for(i=0;i<num_exemplars;i++){
		for(j=0;j<CFTable[0];j++)	fscanf( file, "%lf", &(Training[i].Input[j]) );
		for(j=0;j<CFTable[max_layers-1];j++)	fscanf( file, "%lf", &(Training[i].Target[j]));
	}

	free(s1);
}
	

/* The Routine FileopsByte2FloatRoutine( Widget W, XtPointer client_data, XtPointer call_data ) is in Cantata.c */

	

