#include "SMain.h"
#include "XMain.h"


void	FeedInputs( int exemplar )
/* Will distribute a training input exemplar to the input layer nodes */
/* if exemplar is < 0 it will do the same but with the examination array*/
/* Private */
{
	int 	node;

	for(node=0;node<CFTable[0];node++){
		(NCP[0][node])->input  = 
		(NCP[0][node])->output = 
		(NCP[0][node])->sigmoid= 
			(exemplar < 0   ?   examination[node] : Training[exemplar].Input[node]);
	}
}


void	ForwardPropagate( int exemplar )
/* Will F.propagate a training input exemplar OR examination vector if ex < 0 and end up with final output */
/* External */
{		
	double	input;
	int	lay, node, previous_node;

	FeedInputs( exemplar );

	/* start from lay=1 since input layer has an output */
	for(lay=1;lay<max_layers;lay++)
		for(node=0;node<CFTable[lay];node++){
			input = 0.0;
			for(previous_node=0;previous_node<CFTable[lay-1];previous_node++){
				/* sum the products: pr. layer output * weight */
				input += (NCP[lay-1][previous_node]->sigmoid) * 
					 (NCP[lay][node]->weight)[previous_node];
			} /* input = SUM( pr.out * W connecting curent and pr.node*/
			NCP[lay][node]->input = input;
			NCP[lay][node]->sigmoid= f(input, NCP[lay][node]->threshold);
			NCP[lay][node]->output = input + NCP[lay][node]->threshold;
		}
	/* For the current training exemplar all the nodes in all layers
	   have an output. we have final outputs as well */
}


void	BackPropagate( int exemplar )
/* Will BACK propagate a training input exemplar and end up with all deltas */
/* External */
{		

	double	deltasum;
	int	lay, node, next_node;

	/* Output layer deltas */
	for(node=0;node<CFTable[max_layers-1];node++){
		NCP[max_layers-1][node]->delta = (Training[exemplar].Target[node] -
						  NCP[max_layers-1][node]->sigmoid) *
				   		 (1.0 - NCP[max_layers-1][node]->sigmoid) *
				   		  NCP[max_layers-1][node]->sigmoid;
	}

	/* all other layers deltas */
	for(lay=max_layers-2;lay>0;lay--)
		for(node=0;node<CFTable[lay];node++){
			deltasum = 0.0;
			for(next_node=0;next_node<CFTable[lay+1];next_node++){
				deltasum += ( (NCP[lay+1][next_node]->delta) *
					      (NCP[lay+1][next_node]->weight)[node] );
				/* calculate the sum of the products:
				   next layer delta * weights connecting current node
				   with all nodes in the next layer */
			}
			NCP[lay][node]->delta =  deltasum *
						(1.0 - NCP[lay][node]->sigmoid) *
					    	(NCP[lay][node]->sigmoid);
		}
	
}


void	CalculateNewWeights( void )
/* Will calculate the weight changes and new weights for all nodes in all 
   layers for a training exemplar */
/* External */
{
	Neuron	n;
	double	DW = 0.0;
	int	lay, left_node, this_node,node, previous_node;


	/* for a weight connecting i and j layers we need delta of j and input of j*/
	for(lay=1;lay<max_layers;lay++)
		for(node=0;node<CFTable[lay];node++){
			for(previous_node=0;previous_node<CFTable[lay-1];previous_node++){
	                DW = Beta *
	                     (NCP[lay][node]->delta) *
                             (NCP[lay-1][previous_node]->sigmoid);
			(NCP[lay][node]->weight)[previous_node] = DW +
								  Lamda * 
						 		 (NCP[lay][node]->weight)[previous_node];
			} /* for previous */
		}

}


double	f( double input, double threshold )
/* Will return the output of the neuron who has a total input and a thr. */
/* Private */
{
	double	output;

	output = 1.0/( 1.0 + exp(-input-threshold) );
	return( output );
}


void	PutNoise(void)
{
	        int lay, node, previous_node;

	        FillRandomArray();
	        
                for(lay=1;lay<max_layers;lay++)
	                for(node=0;node<CFTable[lay];node++)
		                for(previous_node=0;previous_node<CFTable[lay-1];previous_node++)
	        	                NCP[lay][node]->weight[previous_node] += Random( (double )(MAX_WEIGHT/2.0) );

	        puts("OK NOISE");

}                                                                                                
